<?php  
/**
 * @author Andrew Embler <andrew@concrete5.org>
 * @copyright  Copyright (c) 2003-2008 Concrete5. (http://www.concrete5.org)
 * @license    http://www.concrete5.org/license/     MIT License
 */

/**
 * A modified version of the image helper to handle cropping in different ways
 * It is not a drop in replacement for the existing helper, it is intended to be
 * used within a specific package
 * @author Andrew Embler <andrew@concrete5.org>
 * @copyright  Copyright (c) 2003-2008 Concrete5. (http://www.concrete5.org)
 * @license    http://www.concrete5.org/license/     MIT License
 * Now includes cropping functionality (thanks to Jordan Lev and Kirk Roberts)
 * Changes (Feb, 2012) by Ryan Hewitt - Mesuva Web Development
 *	- addition of flags to change the way images are cropped
 */

defined('C5_EXECUTE') or die("Access Denied.");
class CropImageHelper {

	/**
	 * Creates a new image given an original path, a new path, a target width and height.
	 * Optionally crops image to exactly match given width and height.
	 * @params string $originalPath, string $newpath, int $width, int $height, string $xcrop, string $ycrop
	 * @return void
	 */		
	public function create($originalPath, $newPath, $width, $height, $xcrop = 'middle', $ycrop = 'middle') {
		// first, we grab the original image. We shouldn't ever get to this function unless the image is valid
		$crop = true;
		$imageSize = @getimagesize($originalPath);
		$oWidth = $imageSize[0];
		$oHeight = $imageSize[1];
		$imageRatio = $oWidth/$oHeight;
		
		if ($width/$height > $imageRatio) {
	       $new_height = $width/$imageRatio;
	       $new_width = $width;
	    } else {
	       $new_width = $height*$imageRatio;
	       $new_height = $height;
	    }

	    $x_mid = $new_width / 2;  //horizontal middle
	    $y_mid = $new_height / 2; //vertical middle
		
		if ($xcrop == 'left') {
		//left
			$posx = 0;
		
		} elseif ($xcrop=='right') {
			// right
			$posx = $new_width - $width;
		} else {
		 	// middle
		 	$posx = ($x_mid-($width/2));
		}
		
		
		if ($ycrop == 'top') {
		//top
			$posy = 0;
		
		} elseif ($ycrop=='bottom') {
			// bottom
			$posy = $new_height - $height;
		} else {
		 	// middle
		 	$posy = ($y_mid-($height/2));
		}
		
		$im = false;		
		switch($imageSize[2]) {
			case IMAGETYPE_GIF:
				$im = @imageCreateFromGIF($originalPath);
				break;
			case IMAGETYPE_JPEG:
				$im = @imageCreateFromJPEG($originalPath);
				break;
			case IMAGETYPE_PNG:
				$im = @imageCreateFromPNG($originalPath);
				break;
		}
		
		if ($im) {
						
			//create "canvas" to put new resized and/or cropped image into
			$image = @imageCreateTrueColor($width, $height);
			$process = @imageCreateTrueColor(round($new_width), round($new_height));
			
			// Better transparency - thanks for the ideas and some code from mediumexposure.com
			if (($imageSize[2] == IMAGETYPE_GIF) || ($imageSize[2] == IMAGETYPE_PNG)) {
				$trnprt_indx = imagecolortransparent($im);
				
				// If we have a specific transparent color
				if ($trnprt_indx >= 0) {
			
					// Get the original image's transparent color's RGB values
					$trnprt_color = imagecolorsforindex($im, $trnprt_indx);
					
					// Allocate the same color in the new image resource
					$trnprt_indx = imagecolorallocate($image, $trnprt_color['red'], $trnprt_color['green'], $trnprt_color['blue']);
					$trnprt_indx2 = imagecolorallocate($process, $trnprt_color['red'], $trnprt_color['green'], $trnprt_color['blue']);
					
					// Completely fill the background of the new image with allocated color.
					imagefill($image, 0, 0, $trnprt_indx);
					imagefill($process, 0, 0, $trnprt_indx2);
					
					// Set the background color for new image to transparent
					imagecolortransparent($image, $trnprt_indx);
					imagecolortransparent($process, $trnprt_indx2);
					
				
				} else if ($imageSize[2] == IMAGETYPE_PNG) {
				
					// Turn off transparency blending (temporarily)
					imagealphablending($image, false);
					imagealphablending($process, false);
					
					// Create a new transparent color for image
					$color = imagecolorallocatealpha($image, 0, 0, 0, 127);
					
					// Completely fill the background of the new image with allocated color.
					imagefill($image, 0, 0, $color);
					imagefill($process, 0, 0, $color);
					
					// Restore transparency blending
					imagesavealpha($image, true);
					imagesavealpha($process, true);
			
				}
			}
			 
			imageCopyResampled($process, $im, 0, 0, 0, 0, $new_width, $new_height, $oWidth, $oHeight);
			
			$res = @imageCopyResampled($image, $process, 0, 0, $posx, $posy, $width, $height, $width, $height);
			if ($res) {
				switch($imageSize[2]) {
					case IMAGETYPE_GIF:
						$res2 = imageGIF($image, $newPath);
						break;
					case IMAGETYPE_JPEG:
						$compression = defined('AL_THUMBNAIL_JPEG_COMPRESSION') ? AL_THUMBNAIL_JPEG_COMPRESSION : 80;
						$res2 = imageJPEG($image, $newPath, $compression);
						break;
					case IMAGETYPE_PNG:
						$res2 = imagePNG($image, $newPath);
						break;
				}
			}
		}
	}
	
	/** 
	 * Returns a path to the specified item, resized and/or cropped to meet max width and height. $obj can either be
	 * a string (path) or a file object. 
	 * Returns an object with the following properties: src, width, height
	 * @param mixed $obj
	 * @param int $maxWidth
	 * @param int $maxHeight
	 * @param string $xcrop
	 * @param string $ycrop
	 */
	public function getThumbnail($obj, $maxWidth, $maxHeight, $xcrop = 'middle', $ycrop = 'middle') {
		$crop = true;
		
		if ($obj instanceof File) {
			$path = $obj->getPath();
		} else {
			$path = $obj;
		}		
		
		$fh = Loader::helper('file');
		$filename = md5('cropped:' .	 $xcrop . ':'. $ycrop . ':'.  $path . ':' . $maxWidth . ':' . $maxHeight . ':' . filemtime($path)) . '.' . $fh->getExtension($path);
		
		
		if (!file_exists(DIR_FILES_CACHE . '/' . $filename)) {
			//create image there
			$this->create($path, DIR_FILES_CACHE . '/' . $filename, $maxWidth, $maxHeight, $xcrop, $ycrop);
		}
		
		$src = REL_DIR_FILES_CACHE . '/' . $filename;
		$abspath = DIR_FILES_CACHE . '/' . $filename;
		$thumb = new stdClass;
		if (isset($abspath) && file_exists($abspath)) {			
			$thumb->src = $src;
			$dimensions = getimagesize($abspath);
			$thumb->width = $dimensions[0];
			$thumb->height = $dimensions[1];
			return $thumb;
		}					
	}
	
	/** 
	 * Runs getThumbnail on the path, and then prints it out as an XHTML image
	 */
	public function outputThumbnail($obj, $maxWidth, $maxHeight, $alt = null, $xcrop = 'middle', $ycrop = 'middle') {
		$crop = true;
		$thumb = $this->getThumbnail($obj, $maxWidth, $maxHeight, $xcrop, $ycrop);
		$html = '<img class="ccm-output-thumbnail" alt="' . $alt . '" src="' . $thumb->src . '" width="' . $thumb->width . '" height="' . $thumb->height . '" />';
		print $html;
		
	}
	

}